// The demonstration game for The Quill Adventure Writer by Zeljko Juric.
// Originally written by Ian Coolier for ZX Spectrum with Gilsoft's The Quill.
// Significantly improved and ported to TI-89/92+ by Zeljko Juric.

// This game is a coding example, which illustrates some more advanced coding
// tricks, like more inteligent parsing of complex sentences, automatic 
// detection of possible movements in the Connections Section, intercepting
// of some movements to make them deadly, usage of two fonts in one game,
// implementation of OOPS or UNDO commands, etc.

$$TITLE("The Castle")

// +-----------------+
// | Aliases Section |
// +-----------------+

// Locations aliases

DEFINE(INTRO1,0)
DEFINE(INTRO2,1)
DEFINE(DARK_PLACE,2)
DEFINE(JUNCTION,3)
DEFINE(LABORATORY,4)
DEFINE(DARK_DUNGEON,5)
DEFINE(DUNGEON,6)
DEFINE(LIT_ROOM,7)
DEFINE(CAVERN,8)
DEFINE(PASSAGE,9)
DEFINE(SQUARE_ROOM,10)
DEFINE(OUTSIDE,11)
DEFINE(LARGE_ROOM,12)
DEFINE(MAGICIAN_ROOM,13)
DEFINE(LONG_ROOM,14)
DEFINE(RED_ROOM,15)
DEFINE(DARK_ROOM,16)

// Objects aliases

DEFINE(TORCH,0)
DEFINE(UNLIT_TORCH,1)
DEFINE(LIT_TORCH,2)
DEFINE(BATTERY,3)
DEFINE(STONE,4)
DEFINE(BONE,5)
DEFINE(GAS_MASK,6)
DEFINE(CANISTER,7)
DEFINE(CHAIR,8)
DEFINE(SPIDERS,9)
DEFINE(DOGS,10)
DEFINE(SCREWDRIVER,11)
DEFINE(BENCH,12)
DEFINE(BOOKCASE,13)
DEFINE(ROPE,14)
DEFINE(TREASURE,15)
DEFINE(KEY,16)
DEFINE(TIED_ROPE,17)

// Flags aliases

DEFINE(BENCH_MOVED,11)                 // Notzero if a bench is moved
DEFINE(ON_CHAIR,12)                    // Notzero while standing on chair
DEFINE(WINDOW_BROKEN,13)               // Notzero if a window is broken
DEFINE(PLATE_UNSCREWED,14)             // Notzero if a plate is unscrewed
DEFINE(BOOKCASE_MOVED,15)              // Notzero if a bookcase is moved
DEFINE(THIRD_WORD,16)                  // Keeps the third recognized word
DEFINE(EXITS,17)                       // Aux variable for detecting exits
DEFINE(NO_OF_EXITS,18)                 // Keeps the number of exits
DEFINE(SAVE_LOC,19)                    // Temporary storage for $CURLOC
DEFINE(SAVE_VERB,20)                   // Temporary storage for $VERB

$$VOCABULARY(4)

// +--------------------+
// | Vocabulary Section |
// +--------------------+

WORD(1,N)
WORD(1,NO)                             // Allow NO (instead N) as an answer...
WORD(1,NORTH)
WORD(2,S)
WORD(2,SOUTH)
WORD(3,E)
WORD(3,EAST)
WORD(4,W)
WORD(4,WEST)
WORD(5,NE)
WORD(6,NW)
WORD(7,SE)
WORD(8,SW)
WORD(9,U)
WORD(9,UP)
WORD(9,CLIMB)
WORD(9,ASCEND)
WORD(10,D)
WORD(10,DOWN)
WORD(10,DESCEND)
WORD(15,Y)
WORD(15,YES)
WORD(21,H)
WORD(21,HELP)
WORD(21,WHO)                           // Make all common questions equal to
WORD(21,HOW)                           //   asking for a help...
WORD(21,WHY)
WORD(21,WHER)
WORD(21,WHAT)
WORD(21,WHEN)
WORD(21,WHICH)
WORD(22,TORCH)
WORD(23,BATTERY)
WORD(24,ROCK)
WORD(24,STONE)
WORD(25,BONE)
WORD(26,MASK)
WORD(27,GAS)
WORD(27,CANISTER)
WORD(28,CHAIR)
WORD(29,SPIDERS)
WORD(30,DOGS)
WORD(31,L)
WORD(31,LOOK)
WORD(32,OP)
WORD(32,OPEN)
WORD(33,ON)
WORD(33,LIGHT)
WORD(34,OFF)
WORD(34,EXTINGUISH)
WORD(35,UNL)
WORD(35,UNLOCK)
WORD(36,PLATE)
WORD(37,BUTTON)
WORD(37,SWITCH)
WORD(38,SCREWDRIVER)
WORD(39,UNSCREW)
WORD(39,REMOVE)
WORD(40,BENCH)
WORD(41,BREAK)
WORD(41,SMASH)
WORD(42,FEED)
WORD(43,MOVE)
WORD(43,PUSH)
WORD(44,TIE)
WORD(45,BOOKCASE)
WORD(45,SHELF)
WORD(45,CASE)
WORD(46,ROPE)
WORD(47,TREASURE)
WORD(48,KEY)
WORD(49,SPRAY)
WORD(50,DOOR)
WORD(51,INSERT)
WORD(52,UNTIE)
WORD(53,WINDOW)
WORD(54,SKELETON)
WORD(100,GET)
WORD(100,TAKE)
WORD(100,PICK)
WORD(101,DROP)
WORD(101,PUT)
WORD(102,THROW)
WORD(103,WEAR)
WORD(104,I)
WORD(104,INVENTORY)
WORD(105,R)
WORD(105,REDESCRIBE)
WORD(106,QUIT)
WORD(106,STOP)
WORD(107,SAVE)
WORD(108,LOAD)
WORD(108,RESTORE)
WORD(109,EXAMINE)
WORD(150,BIG)
WORD(151,SMALL)
WORD(152,FONT)
WORD(160,UNDO)
WORD(160,OOPS)
WORD(250,IN)                           // Prepositions...
WORD(250,INTO)
WORD(251,WITH)
WORD(252,AT)

$$LOCATIONS

// +-------------------+
// | Locations Section |
// +-------------------+

LOC($INTRO1," Original by Ian Coolier\n"
    " Adapted by Zeljko Juric\n\n"
    " Press any key...")
LOC($INTRO2,"The object of this adventure is to discover the hidden treasure "
    "in this castle, into which you have just wandered. In the treasure is "
    "the key to the door which has just slammed behind you. You will find "
    "many rooms containing items which the previous occupants (e.g. the "
    "alchemist) left behind. Your task is completed when you go back through "
    "the door you entered. The adventure may end before this, though.\n\n"
    "Good luck (you'll need it)!\n")
LOC($DARK_PLACE,"You are in a dark place. Behind you is the door you entered "
    "at. The only exit is West.")
LOC($JUNCTION,"You are at a junction in the passageway.")
LOC($LABORATORY,"You are in the alchemist's old laboratory.")
LOC($DARK_DUNGEON,"You are in a dark dungeon. There is a little light, but "
    "you can't see.")
LOC($DUNGEON,"You are in a dungeon. There is a window on the east side, too "
    "high to reach. You see a skeleton.")
LOC($LIT_ROOM,"You are in a well-lit room with a bench near the west wall.")
LOC($CAVERN,"You are in a cavern. There are rock formations all around.")
LOC($PASSAGE,"You are in a dark secret passage which leads underground.")
LOC($SQUARE_ROOM,"You are in a square room which looks as if it has been cut "
    "out of rock.")
LOC($OUTSIDE,"You are outside the dungeon. To the west is the window.")
LOC($LARGE_ROOM,"You are in a large room with bare walls.")
LOC($MAGICIAN_ROOM,"You are in the magician's room.")
LOC($LONG_ROOM,"You are in a long room. Secret doors are North and South.")
LOC($RED_ROOM,"You are in a room which had been painted red.")
LOC($DARK_ROOM,"You are in a small, dark room.")

$$MESSAGES

// +------------------+
// | Messages Section |
// +------------------+

MSG(0,"You see nothing special.")
MSG(1,"The dogs attack you and kill you.")
MSG(2,"Crunch crunch!")
MSG(3,"Something drops from above and stings. You are dead.")
MSG(4,"You spray gas. The spiders go away.")
MSG(5,"You spray gas. The gas is poisonous and kills you.")
MSG(6,"You can reach the window now.")
MSG(7,"You can see a passage leading West.")
MSG(8,"There is a secret passase to the South.")
MSG(9,"There is a secret passage to the North.")
MSG(10,"I think you should have used a rope...\n\n"
    "You fell into the dungeon from the high window and broke your neck.")
MSG(11,"You can see some sort of plate, in the centre of which is a screw.")
MSG(12,"You unscrew the plate. The plate falls and an opening forms in the "
    "wall.")
MSG(13,"The bench slides easily across the floor.")
MSG(14,"It says \"For Ye Spiders\".")
MSG(15,"You put the key into its hole and turn. There is a rumbling sound and "
    "a large door opens before you. You go through the door and are free.\n\n"
    "Well done!")
MSG(16,"I have no time for your silly questions. You are supposed to be "
    "solving this adventure yourself!")
MSG(17,"Please say Yes or No!")
MSG(18,"OK. Remember that you can type SMALL FONT later to switch to the "
    "small font.\n\n"
    "Any key to start...")
MSG(19,"OK. Remember that you can type BIG FONT later to switch to the big "
    "font.\n\n"
    "Any key to start...")
MSG(20,"How exactly!?")
MSG(21,"Impossible, while you are carrying it...")
MSG(22,"You fell from the chair and broke your neck.")
MSG(23,"You can not reach the window.")
MSG(24,"It is full of bones, which is not unusual for a skeleton, isn't it?")
MSG(25,"The secret passage is blocked.")
MSG(26,"\nNote, however, that this is not the only solution for the game. "
    "For example, ")
MSG(27,"have you tried to pass the spiders?")
MSG(28,"have you tried to pass the dogs?")

// The following messages are used in the subroutine (which is coded inside
// the Events Section) which autodetects possible exits from a location.

MSG(29,"Exit:")
MSG(30,"Exits:")
MSG(31," North")
MSG(32," South")
MSG(33," East")
MSG(34," West")
MSG(35," NorthEast")
MSG(36," NorthWest")
MSG(37," SouthEast")
MSG(38," SouthWest")

$$SYSTEM_MESSAGES

// +-------------------------+
// | System Messages Section |
// +-------------------------+

MSG(0,"Everything is dark. I can't see.")
MSG(1,"You can also see:")
MSG(2,"\nI await your command.")
MSG(3,"\nI'm ready for your instructions.")
MSG(4,"\nTell me what to do.")
MSG(5,"\nGive me your command.")
MSG(6,"Sorry, I don't understand that. Try some different words.")
MSG(7,"You can't go in that direction.")
MSG(8,"You can't do that.")
MSG(9,"You are carried:")
MSG(10," (worn)")
MSG(11,"Nothing at all.")
MSG(12,"Do you really want to quit now?")
MSG(13,"\nEnd of game. Try again?")
MSG(14,"\nGoodbye...")
MSG(15,"OK.")
MSG(16,"")
MSG(17,"You have taken ")
MSG(18," turn")
MSG(19,"s")
MSG(20,".")
MSG(23,"You're not wearing it.")
MSG(24,"You can't. Your hands are full.")
MSG(25,"You already have it.")
MSG(26,"It's not here.")
MSG(27,"You can't carry any more.")
MSG(28,"You don't have it.")
MSG(29,"You're already wearing it.")
MSG(30,"Y")
MSG(31,"N")

// Extra prompts...

MSG(32,"Do you want big font?")
MSG(33,"Into what?")
MSG(34,"With what?")
MSG(35,"At what?")

$$OBJECTS(6)                           // The player can carry 6 objects

// +-----------------+
// | Objects Section |
// +-----------------+

// Note that $TORCH is an "unqualified" torch, which exists only at the
// begining on the game. Once taken, it will become $UNLIT_TORCH. Check the
// Actions Section to see how this is implemented exactly.

OBJ($TORCH,"A torch that a previous explorer left behind.",_,$JUNCTION)
OBJ($UNLIT_TORCH,"A torch.",_,$NOWHERE)
OBJ($LIT_TORCH,"A lit torch.",TORCH,$NOWHERE)
OBJ($BATTERY,"A torch battery.",BATTERY,$LABORATORY)
OBJ($STONE,"A stone.",STONE,$CAVERN)
OBJ($BONE,"A bone.",BONE,$NOWHERE)
OBJ($GAS_MASK,"A sort of gas mask.",MASK,$PASSAGE)
OBJ($CANISTER,"A canister of gas.",CANISTER,$LABORATORY)
OBJ($CHAIR,"An old chair.",CHAIR,$LIT_ROOM)
OBJ($SPIDERS,"Some spiders.",_,$RED_ROOM)
OBJ($DOGS,"Some dogs.",_,$LARGE_ROOM)
OBJ($SCREWDRIVER,"A small screwdriver.",SCREWDRIVER,$SQUARE_ROOM)
OBJ($BENCH,"A bench.",_,$NOWHERE)
OBJ($BOOKCASE,"A bookcase.",_,$MAGICIAN_ROOM)
OBJ($ROPE,"A rope.",ROPE,$LONG_ROOM)
OBJ($TREASURE,"The treasure.",TREASURE,$LONG_ROOM)
OBJ($KEY,"The Key.",KEY,$LONG_ROOM)
OBJ($TIED_ROPE,"A tied rope.",_,$NOWHERE)

$$CONNECTIONS

// +---------------------+
// | Connections Section |
// +---------------------+

FROM($DARK_PLACE)
    CONN(W,$JUNCTION)
    
FROM($JUNCTION)
    CONN(E,$DARK_PLACE)
    CONN(S,$CAVERN)
    CONN(SW,$LIT_ROOM)
    CONN(NW,$LABORATORY)
    CONN(NE,$DARK_DUNGEON)
    
FROM($LABORATORY)
    CONN(SE,$JUNCTION)
    
FROM($DARK_DUNGEON)
    CONN(SE,$JUNCTION)
    
FROM($DUNGEON)
    CONN(SE,$JUNCTION)
    
FROM($LIT_ROOM)
    CONN(NE,$JUNCTION)
    
FROM($CAVERN)
    CONN(N,$JUNCTION)
    
FROM($PASSAGE)
    CONN(W,$SQUARE_ROOM)
    CONN(E,$LIT_ROOM)
    
FROM($SQUARE_ROOM)
    CONN(E,$PASSAGE)
    
FROM($OUTSIDE)
    CONN(S,$LARGE_ROOM)
    CONN(N,$DARK_ROOM)

FROM($LARGE_ROOM)
    CONN(N,$OUTSIDE)
    CONN(E,$MAGICIAN_ROOM)

FROM($MAGICIAN_ROOM)
    CONN(W,$LARGE_ROOM)
    
FROM($RED_ROOM)
    CONN(W,$DARK_ROOM)
    
FROM($DARK_ROOM)
    CONN(E,$RED_ROOM)
    CONN(S,$OUTSIDE)
    
$$EVENTS

// +----------------+
// | Events Section |
// +----------------+

// These two entries must be before the location description entries

EVENT
    AT($INTRO1)                        // At the very begining, choose the big
    BIGFONT                            //   font (before the loc is described)

EVENT
    AT($DARK_DUNGEON)                  // Dark dungeon in presence of the lit
    CARRIED($LIT_TORCH)                //   torch should become an ordinary
    GOTO($DUNGEON)                     //   dungeon...

// Now follows a subroutine made of tricky and somewhat awkward entries for
// automatic detection and displaying possible exits. This is useful if you
// don't want to include the descriptions of possible exits in the 
// descriptions of the locations themselves. So, this can save a file space
// if there is a lot of locations in the game (the subroutine itself takes 
// about 150 bytes). However, this makes the descriptions of the locations
// more stereotipe and less interesting. If you don't understand quite well
// this subroutine, you can use it as-is in your games...

EVENT
    CLEAR($EXITS)                      // Clear information about exits
    CLEAR($NO_OF_EXITS)                // Set number of exits to 0 so far
    COPYFF($CURLOC,$SAVE_LOC)          // Save the current location
    COPYFF($VERB,$SAVE_VERB)           // Save the value of $VERB flag
    SETVERB(N)                         // Set verb to N
    TRYMOVE                            // Try to move north
    COPYFF($SAVE_LOC,$CURLOC)          // If possible, restore the original
    PLUS($NO_OF_EXITS,1)               //   location, increase the number of
    PLUS($EXITS,128)                   //   exits, and signal "can go N"...

EVENT
    SETVERB(S)                         // The similar entries follows for
    TRYMOVE                            //   each possible direction...
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,64)                    // Signal "can go S"...

EVENT 
    SETVERB(E)
    TRYMOVE
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,32)                    // Signal "can go E"...

EVENT 
    SETVERB(W)
    TRYMOVE
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,16)                    // Signal "can go W"...

EVENT 
    SETVERB(NE)
    TRYMOVE
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,8)                     // Signal "can go NE"...

EVENT 
    SETVERB(NW)
    TRYMOVE
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,4)                     // Signal "can go NW"...

EVENT 
    SETVERB(SE)
    TRYMOVE
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,2)                     // Signal "can go SE"...

EVENT 
    SETVERB(SW)
    TRYMOVE
    COPYFF($SAVE_LOC,$CURLOC)
    PLUS($NO_OF_EXITS,1)
    PLUS($EXITS,1)                     // Signal "can go SW"...

EVENT
    COPYFF($SAVE_VERB,$VERB)           // Restore the verb

// In this game, we want to ignore completely collected information if the
// player is at the dark place, or inside the dark dungeon. This is specific
// for this game, and should not be counted as a part of the subroutine.

EVENT
    AT($DARK_PLACE)                    // Dark place?
    CLEAR($NO_OF_EXITS)                // Ignore collected info

EVENT
    AT($DARK_DUNGEON)                  // Dark dungeon?
    CLEAR($NO_OF_EXITS)                // Ignore collected info

// Now, information about possible exits are collected. We need to hook into
// the Location-Display entry to display collected information.

EVENT
    ISDESC                             // Description in progress?
    CLS                                // Clear screen
    SHOWLOC                            // Display the location
    NEWLINE
    
EVENT
    ISDESC                             // Description in progress?
    ZERO($NO_OF_EXITS)                 // No exits?
    LISTOBJ                            // List objects
    ENDDESC                            // Signal "description is done"

EVENT
    ISDESC                             // Description in progress?
    EQ($NO_OF_EXITS,1)                 // Just one exit?
    MES(29)                            // Exit:

EVENT
    ISDESC                             // Description in progress?
    GT($NO_OF_EXITS,1)                 // More than one exit?
    MES(30)                            // Exits:

// Information about exits are collected in $EXIT flag, which is, in fact, a
// collection of binary indicators. As there is no bitwise operations in the
// QADL, we must to simulate them. This is not hard if you check bits starting
// from the most significant bit. 

EVENT
    ISDESC                             // Description in progress?
    GT($EXITS,127)                     // Can go north?
    MES(31)                            // North
    MINUS($EXITS,128)                  // Reset the indicator

EVENT
    ISDESC                             // Similar entries follows for each
    GT($EXITS,63)                      // possible direction...
    MES(32)                            // South
    MINUS($EXITS,64)

EVENT
    ISDESC
    GT($EXITS,31)
    MES(33)                            // East
    MINUS($EXITS,32)
    
EVENT
    ISDESC
    GT($EXITS,15)
    MES(34)                            // West
    MINUS($EXITS,16)

EVENT
    ISDESC
    GT($EXITS,7)
    MES(35)                            // NorthEast
    MINUS($EXITS,8)

EVENT
    ISDESC
    GT($EXITS,3)
    MES(36)                            // NorthWest
    MINUS($EXITS,4)

EVENT
    ISDESC
    GT($EXITS,1)
    MES(37)                            // SouthEast
    MINUS($EXITS,2)

EVENT
    ISDESC
    NOTZERO($EXITS)
    MES(38)                            // SouthWest

EVENT
    ISDESC                             // Description in progress?
    NEWLINE                            // List objects (finally)
    LISTOBJ
        
// The location is now fully described. The "describe location" subroutine is
// finished, so the tricky part of the Events Section is over...

// Now follows a set of less or more ordinary entries...

EVENT    
    AT($INTRO1)                        // First intro screen?
    ANYKEY                             // Wait for a keypress...
    GOTO($INTRO2)                      // Prepare the second intro screen
    SMLFONT                            // Switch to small font
    PROMPT(32)                         // "Do you want big font?" prompt    
    DESC                               // Show the second intro screen
    
EVENT
    PRESENT($SPIDERS)                  // Spiders?
    ZERO($CNT1)                        // Three moves before the death...
    LET($CNT1,3)

EVENT
    EQ($CNT1,1)                        // Time expired (spiders)?
    MESSAGE(3)                         // Something drops from above...
    TURNS                              // Bad luck...
    END
    
EVENT
    AT($RED_ROOM)                      // Red room?
    NOTZERO($PLATE_UNSCREWED)          // Plate unscrewed?
    MESSAGE(8)                         // Secret passage to south...
    
EVENT
    AT($LIT_ROOM)                      // Lit room?
    NOTZERO($BENCH_MOVED)              // Bench moved?
    MESSAGE(7)                         // A passage to west...
    
EVENT
    AT($MAGICIAN_ROOM)                 // Magician room?
    NOTZERO($BOOKCASE_MOVED)           // Bookcase moved?
    MESSAGE(9)                         // Secret passage to north...
    
$$ACTIONS

// +-----------------+
// | Actions Section |
// +-----------------+

// "OOPS" or "UNDO" entries, if implemented, should always be the very first
// entries in the Actions Section.

ACTION(OOPS,_)
    RAMLOAD                            // Restore the state...
    DESC                               // Redescribe...
    
// Entries for accepting "YES" or "NO" answers to the question about fonts...

ACTION(YES,_)
    AT($INTRO2)                        // Second intro screen?
    MES(18)                            // Well, big font will be used
    ANYKEY                             // Wait for a keypress
    BIGFONT                            // Use big font
    PROMPT(0)                          // Restore the standard prompt
    GOTO($DARK_PLACE)                  // The game begins at the dark place
    DESC                               // Describe it...

ACTION(NO,_)
    AT($INTRO2)                        // Second intro screen?
    MES(19)                            // Well, small font will be used
    ANYKEY                             // Wait for a keypress
    SMLFONT                            // Use big font
    PROMPT(0)                          // Restore the standard prompt
    GOTO($DARK_PLACE)                  // The game begins at the dark place
    DESC                               // Describe it...

ACTION(_,_)                            // Anything else (not YES or NO)?
    AT($INTRO2)                        // Second intro screen?
    MESSAGE(17)                        // Answer YES or NO...
    DONE

// Now, it's the time to process responses to questions caused by incomplete
// sentences. This should be done before processing normal ones.

// "INSERT BATTERY" without "IN" or "INTO" preposition sets "Into what?" 
// prompt. Now, process responses to it...

ACTION(IN,TORCH)
    EQ($PROMPT,33)                     // "Into what?" prompt?
    PRESENT($UNLIT_TORCH)              // Unlit torch is here?
    DESTROY($BATTERY)                  // No more battery as a single object
    SWAP($UNLIT_TORCH,$LIT_TORCH)      // Make a torch lit
    PROMPT(0)                          // Restore the standard prompt
    OK

// "BREAK WINDOW" without "WITH" preposition sets "With what?" prompt...

ACTION(WITH,STONE)
    EQ($PROMPT,34)                     // "With what?" prompt?
    CARRIED($STONE)                    // Stone carried?
    SET($WINDOW_BROKEN)                // Well, now the window is broken
    DROP($STONE)                       // And, drop the stone (of course)...    
    PROMPT(0)                          // Restore the standard prompt
    OK

// In the dungeon, "THROW STONE" without "AT" preposition sets "At what?" 
// prompt...

ACTION(AT,WINDOW)
    EQ($PROMPT,35)                     // "At what?" prompt?
    SET($WINDOW_BROKEN)
    PROMPT(0)
    OK

// Any other response to "Into what?", "With what?" and "At what?" is
// meaningless in this game...
    
ACTION(_,_)
    GT($PROMPT,31)                     // A special prompt?
    PROMPT(0)                          // Switch back to the standard prompt
    SYSMESS(8)                         // You can't (note that usage of
    NEWLINE                            //   NOTDONE would produce different
    DONE                               //   response to movement words)...

// The processing of incomplete sentences is now finished.

// Support for "OOPS" and "UNDO". Think about why it is inserted just here.

ACTION(_,_)
    RAMSAVE
    CANCEL

// Extract the third recognized word (if any). It will be useful for some
// actions later.

ACTION(_,_)
    GETWORD($THIRD_WORD)               // Pick the third word
    CANCEL                             // Do not count GETWORD as 'action'

// Now, we start coding of complete sentences (movement words are treated as
// complete sentences too).

ACTION(FEED,DOGS)
    PRESENT($DOGS)                     // Dogs are here?
    CARRIED($BONE)                     // Bone carried?
    DESTROY($BONE)                     // Goodbye bone...
    DESTROY($DOGS)                     // Goodbye dogs (wow)...
    MESSAGE(2)                         // Crunch, crunch...

ACTION(_,_)                            // Anything else is deadly in
    PRESENT($DOGS)                     // presence of dogs...
    MESSAGE(1)
    TURNS
    END

// Movements: any movement while standing on the chair is deadly, except
// moving through the window from the dungeon...

ACTION(E,_)
    AT($DUNGEON)                       // Dungeon?
    NOTZERO($ON_CHAIR)                 // Standing on chair?
    NOTZERO($WINDOW_BROKEN)            // Window broken?
    CLEAR($ON_CHAIR)                   // No more on chair...
    GOTO($OUTSIDE)                     // Well, go east...
    DESC

ACTION(D,_)
    NOTZERO($ON_CHAIR)                 // Standing on chair?
    CLEAR($ON_CHAIR)                   // Mo more...
    OK

ACTION(_,_)
    LT($VERB,13)                       // Movement word?
    NOTZERO($ON_CHAIR)                 // Standing on chair?
    MESSAGE(22)                        // Fall from chair...
    TURNS                              // Bad luck!
    END
  ELSE
    TRYMOVE                            // Try to perform standard movements
    DESC                               //   through the Connections Section

// Other non-standard (conditional) movements...

ACTION(N,_)
    AT($LONG_ROOM)                     // Long room?
    NOTZERO($PLATE_UNSCREWED)          // Plate unscrewed?
    GOTO($RED_ROOM)                    // Well, go...
    DESC
  ELSE
    AT($LONG_ROOM)                     // Otherwise: Long room (but plate not
    MESSAGE(25)                        //   unscrewed)?
    DONE                               // Secret passage is blocked...
  ELSE
    AT($MAGICIAN_ROOM)                 // Magician room?
    NOTZERO($BOOKCASE_MOVED)           // Bookcase moved?
    GOTO($LONG_ROOM)                   // Well, go...
    DESC
    
ACTION(S,_)
    AT($LONG_ROOM)                     // Long room?
    NOTZERO($BOOKCASE_MOVED)           // Bookcase moved?
    GOTO($MAGICIAN_ROOM)               // Well, go...
    DESC
  ELSE
    AT($LONG_ROOM)                     // Otherwise: Long room (but bookcase
    MESSAGE(25)                        //   not moved)?
    DONE                               // Secret passage is blocked...
  ELSE
    AT($RED_ROOM)                      // Red room?
    NOTZERO($PLATE_UNSCREWED)          // Plate unscrewed?
    GOTO($LONG_ROOM)                   // Well, go...
    DESC
    
ACTION(W,_)
    AT($LIT_ROOM)                      // Lit room?
    NOTZERO($BENCH_MOVED)              // Bench moved?
    GOTO($PASSAGE)                     // Well, go...
    DESC
  ELSE
    AT($OUTSIDE)                       // Outside?
    PRESENT($TIED_ROPE)                // Tied rope is here?
    GOTO($DUNGEON)                     // Well, go...
    DESC
  ELSE
    AT($OUTSIDE)                       // Otherwise: Outside (without rope)?
    MESSAGE(10)                        // You fall...
    TURNS                              // Bad luck!
    END

// We want to make LOOK equal to REDESCRIBE, but LOOK _object_ equal to
// EXAMINE _object_ ...
    
ACTION(LOOK,_)
    SETVERB(REDESCRIBE)                // Set verb to REDESCRIBE
    ISNOTNULL($NOUN)                   // Noun recognized?
    SETVERB(EXAMINE)                   // If it is, set verb to EXAMINE
  CONTINUE
    CANCEL                             // Signal "nothing concrete so far..."

// Some very common entries...

ACTION(GET,I)
    INVEN

ACTION(I,_)
    INVEN
    
ACTION(R,_)
    DESC
    
ACTION(QUIT,_)
    QUIT
    NEWLINE
    TURNS
    END
    
ACTION(SAVE,_)
    SAVE
    
ACTION(LOAD,_)
    LOAD
    
ACTION(BIG,FONT)                       // These two entries are not common,
    BIGFONT                            //   but they are mentioned at the
    DESC                               //   introduction page of the game...
    
ACTION(SMALL,FONT)
    SMLFONT
    DESC        

ACTION(SPRAY,SPIDERS)
    CARRIED($CANISTER)                 // Canister carried?
    NOTWORN($GAS_MASK)                 // Gas mask not worn?
    MESSAGE(5)                         // The gas is poisonous...
    TURNS                              // Bad luck...
    END
  ELSE
    CARRIED($CANISTER)                 // Otherwise: Canister carried and
    PRESENT($SPIDERS)                  //   spiders here (with worn mask)?
    MESSAGE(4)                         // You spray...
    DESTROY($SPIDERS)                  // Goodbye spiders...
    CLEAR($CNT1)                       // Reset the deadly counter...
    
ACTION(TORCH,ON)
    PRESENT($UNLIT_TORCH)              // Unlit torch is here?
    MESSAGE(20)                        // How exactly!?

ACTION(ON,TORCH)
    PRESENT($UNLIT_TORCH)              // Unlit torch is here?
    MESSAGE(20)                        // How exactly!?

ACTION(TORCH,OFF)
    PRESENT($LIT_TORCH)                // Lit torch is here?
    MESSAGE(20)                        // How exactly!?

ACTION(OFF,TORCH)
    PRESENT($LIT_TORCH)                // Lit torch is here?
    MESSAGE(20)                        // How exactly!?

ACTION(INSERT,BATTERY)
    NEQWORD($THIRD_WORD,INTO)          // No prepositions IN or INTO?
    PRESENT($BATTERY)                  // Battery is here?
    PROMPT(33)                         // Set prompt to "Into what?"
    DONE
  ELSE
    EXTWORD(IN)                        // INSERT BATTERY IN TORCH?
    EXTWORD(TORCH)
    PRESENT($UNLIT_TORCH)              // Unlit torch is here?
    PRESENT($BATTERY)                  // Battery is here?
    DESTROY($BATTERY)                  // No more battery as a single object
    SWAP($UNLIT_TORCH,$LIT_TORCH)      // Make a torch lit
    OK
    
ACTION(REMOVE,BATTERY)
    PRESENT($LIT_TORCH)                // Lit torch is here?
    SWAP($UNLIT_TORCH,$LIT_TORCH)      // Well, it would not be lit anymore...
    CREATE($BATTERY)                   // Create battery as a separate object
    GET($BATTERY)                      // Get it...
    OK
    
ACTION(EXAMINE,GAS)
    PRESENT($CANISTER)                 // Canister is here?
    MESSAGE(14)                        // For spiders...
    DONE
    
ACTION(EXAMINE,BENCH)
    AT($LIT_ROOM)                      // Lit room?
    MESSAGE(13)                        // It slides...
    DONE

ACTION(EXAMINE,SKELETON)
    AT($DUNGEON)                       // Dungeon?
    MESSAGE(24)                        // Full of bones...
    DONE
    
ACTION(EXAMINE,_)                      // Examine anything other?
    AT($RED_ROOM)                      // Red room?
    MESSAGE(11)                        // Plate and screw...
    DONE
  ELSE
    MESSAGE(0)                         // Nothing special...

ACTION(UNLOCK,DOOR)
    AT($DARK_PLACE)                    // Dark place?
    CARRIED($KEY)                      // Key carried?
    MESSAGE(15)                        // Well done!
    TURNS
    NOTCREATED($DOGS)                  // Dogs puzzle solved?
    NOTCREATED($SPIDERS)               // Spiders puzzle solved?
    END                                // Ok, you are expert...
  ELSE
    AT($DARK_PLACE)                    // Ohterwise: Dark place and key
    CARRIED($KEY)                      //   carried (both puzzles not solved)?
    MES(26)                            // Well done! But...
    CREATED($SPIDERS)                  // Spiders puzzle not solved?
    MESSAGE(27)                        // Try to solve it next time...
    END
  ELSE
    AT($DARK_PLACE)                    // Otherwise: Dark place and key
    CARRIED($KEY)                      //   carried (spiders puzzle solved
    MESSAGE(28)                        //   but dog puzzle not solved)? 
    END                                // Try to solve it next time...

ACTION(UNSCREW,PLATE)
    AT($RED_ROOM)                      // Red room?
    CARRIED($SCREWDRIVER)              // Screwdriver carried?
    ZERO($PLATE_UNSCREWED)             // Plate not unscrewed yet?
    SET($PLATE_UNSCREWED)              // Well, now it is...
    MESSAGE(12)                        // You made an opening...

ACTION(MOVE,BENCH)
    AT($LIT_ROOM)                      // Lit room?
    ZERO($BENCH_MOVED)                 // Bench not moved?
    SET($BENCH_MOVED)                  // Well, move it...
    OK
  ELSE
    AT($LIT_ROOM)                      // Otherwise: Lit room (bench moved)?
    CLEAR($BENCH_MOVED)                // Move the bench back.
    OK
    
ACTION(MOVE,BOOKCASE)
    AT($MAGICIAN_ROOM)                 // Magician room?
    ZERO($BOOKCASE_MOVED)              // Bookcase not moved?
    SET($BOOKCASE_MOVED)               // Well, move it...
    OK

ACTION(ON,CHAIR)
    SETVERB(CLIMB)                     // Transform ON CHAIR to CLIMB CHAIR
    CANCEL                             // Don't count this as an 'action'...
    
ACTION(CLIMB,CHAIR)
    CARRIED($CHAIR)                    // Chair carried?
    MESSAGE(21)                        // Silly intention...
    DONE
  ELSE
    ABSENT($CHAIR)                     // Chair not available?
    SYSMESS(26)                        // It's not here.
    NEWLINE
    DONE
  ELSE
    SET($ON_CHAIR)                     // Signal "standing on chair"...
    AT($DUNGEON)                       // Dungeon?
    MESSAGE(6)                         // You can now reach the window...
    DONE
  ELSE
    OK                                 // Display OK if not in dungeon...
    
ACTION(BREAK,WINDOW)
    AT($DUNGEON)                       // Dungeon?
    ZERO($WINDOW_BROKEN)               // Window not broken?    
    ZERO($ON_CHAIR)                    // Not standing on chair?
    MESSAGE(23)                        // Too high...
    DONE
  ELSE
    AT($DUNGEON)                       // Otherwise: In dungeon and window not
    ZERO($WINDOW_BROKEN)               //   broken (but standing on chair)?
    CARRIED($STONE)                    // Stone carried?    
    EXTWORD(WITH)                      // Phrase BREAK WINDOW WITH STONE?
    EXTWORD(STONE)
    SET($WINDOW_BROKEN)                // Now, the window is broken...
    OK
  ELSE    
    AT($DUNGEON)                       // Otherwise: in dungeon and window not
    ZERO($WINDOW_BROKEN)               //   broken...
    NEQWORD($THIRD_WORD,WITH)          // No preposition WITH?
    PROMPT(34)                         // Set prompt to "With what?"
        
ACTION(TIE,ROPE)
    AT($OUTSIDE)                       // Outside?
    CARRIED($ROPE)                     // Rope carried?
    CREATE($TIED_ROPE)                 // Create tied rope here
    DESTROY($ROPE)                     // Destroy (untied) rope
    OK
    
ACTION(UNTIE,ROPE)                     // Note that SWAP should not be used in
    PRESENT($TIED_ROPE)                //   previous entry, because it would
    SWAP($ROPE,$TIED_ROPE)             //   make tied rope carried...
    OK
    
ACTION(GET,TORCH)
    PRESENT($TORCH)                    // A torch (unqualified) is here?
    DESTROY($TORCH)                    // Ok, no more unqualified torches...
    CREATE($UNLIT_TORCH)               // It is now an "unlit torch"...
    GET($UNLIT_TORCH)                  // Get it...
    OK

ACTION(GET,CHAIR)
    NOTZERO($ON_CHAIR)                 // Standing on chair?
    NOTDONE                            // Silly...
    
ACTION(GET,ROPE)
    PRESENT($TIED_ROPE)                // Tied rope is present?
    NOTDONE                            // Sorry, untie it first...
    
ACTION(GET,BONE)
    AT($DUNGEON)                       // Skeleton is here (in dungeon)?
    CREATE($BONE)                      // Create bone & continue through AUTOG
        
ACTION(GET,_)
    AUTOG                              // Get object, if possible...
    OK

ACTION(THROW,STONE)
    AT($DUNGEON)                       // Dungeon?
    ZERO($WINDOW_BROKEN)               // Window not broken?
    CARRIED($STONE)                    // Stone carried
    EXTWORD(AT)                        // Phrase THROW STONE AT WINDOW?
    EXTWORD(WINDOW)
    SET($WINDOW_BROKEN)                // Well, break the window...
    DROP($STONE)                       // And, drop the stone (of course)...
    OK
  ELSE    
    AT($DUNGEON)                       // Otherwise: In dungeon, stone carried
    CARRIED($STONE)                    //   and window not broken)?
    ZERO($WINDOW_BROKEN)    
    NEQWORD($THIRD_WORD,AT)            // No preposition AT?
    PROMPT(35)                         // Set prompt to "At what?"
    DONE

ACTION(THROW,_)
    SETVERB(DROP)                      // Otherwise, THROW should work as DROP
    CANCEL
        
ACTION(DROP,TORCH)
    CARRIED($UNLIT_TORCH)              // Unlit torch carried?
    DROP($UNLIT_TORCH)                 // Drop it (lit torch will be processed
    OK                                 //   through AUTOD)
    
ACTION(DROP,_)
    AUTOD                              // Drop object, if possible...
    OK
    
ACTION(WEAR,MASK)
    WEAR($GAS_MASK)                    // Wear mask, if possible...
    OK

ACTION(REMOVE,MASK)
    REMOVE($GAS_MASK)                  // Remove mask, if possible...
    OK

ACTION(HELP,_)
    MESSAGE(16)                        // Solve it by yourself...
    
$$PICTURES

// +------------------+
// | Pictures Section |
// +------------------+

// This part is just a cosmetic feature, so you can ignore it completely
// if you're not familiar with C language

CBLOCK($INTRO1,35)
    int old_font=FontSetSys(F_8x10); 
    int xpos=(LCD_WIDTH-144)/2;
    DrawStr(xpos,8,"*** THE CASTLE ***",A_NORMAL);
    DrawClipRect(MakeWinRect(xpos-3,4,xpos+147,20),ScrRect,
      A_NORMAL|B_ROUNDED);
    DrawClipRect(MakeWinRect(xpos-6,1,xpos+150,23),ScrRect,
      A_NORMAL|B_ROUNDED);    
    FontSetSys(old_font);
END_CBLOCK    

$$EXTERN

// +------------------+
// | External Section |
// +------------------+

// No external modules...

$$END
